"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var visitor_wrapper_1 = require("../../utils/visitor-wrapper");
var types_1 = require("@babel/types");
var ERR = new Error("something goes wrong.");
var REQUIRE_CALLEE = types_1.identifier("require");
/**
 * Transform variables defined in ModuleFunction params to their real values
 *
 * minified ModuleFunction: `function(e,t,n){`,
 * unminified: `function(module, exports, __webpack_require__) {`.
 *
 * replace `e` with `module`, `t` with `exports`, `n` -> `__webpack_require__` in extracted program body,
 * and transform `__webpack_require__` to normal `require`
 */
exports.getModuleFunctionParamsTransformer = function (entryId) {
    return visitor_wrapper_1.VisitorWrapper({
        Program: function (path) {
            var node = path.node;
            var paramsTransE = node.body[0];
            if (!types_1.isVariableDeclaration(paramsTransE, { kind: "const" })) {
                throw ERR;
            }
            paramsTransE.declarations.forEach(function (d) {
                var id = d.id, init = d.init;
                if (!types_1.isIdentifier(id) || !types_1.isIdentifier(init)) {
                    throw ERR;
                }
                var paramName = id.name;
                var realValue = init.name;
                path.scope.rename(paramName, realValue);
            });
            // remove paramsTransE
            node.body.shift();
        },
        // transform `__webpack_require__` to normal `require`
        // `__webpack_require__(0)` -> `require("./0")`
        CallExpression: function (path) {
            var node = path.node;
            var callee = node.callee, requireIdE = node.arguments[0];
            // check __webpack_require__
            if (!types_1.isIdentifier(callee, { name: "__webpack_require__" })) {
                return;
            }
            if (!types_1.isNumericLiteral(requireIdE)) {
                throw ERR;
            }
            var requireId = requireIdE.value;
            var isEntryRequire = requireId == entryId;
            var requirePath = "./" + (isEntryRequire ? "entry_" : "") + requireId;
            path.replaceWith(types_1.callExpression(REQUIRE_CALLEE, [types_1.stringLiteral(requirePath)]));
        },
    });
};
